Imports System.Xml
''' <summary>
''' Sammlung verwendeter MP3-Dateien (Sequenzen)
''' </summary>
Public Class MP3Source
    Inherits List(Of MP3File)

#Region " Fields / Events "
    ''' <summary>
    ''' Globaler Ausgabe-Pfad fr MP3-Dateien
    ''' </summary>
    Private Shared m_OutputPath As String

    ''' <summary>
    ''' Globaler Mutex fr Threadsicheres Erstellen von Dateinamen
    ''' </summary>
    Private Shared m_Mutex As New Threading.Mutex()

    ''' <summary>
    ''' globale Liste mit bekannten Dateinamen fr MP3-Dateien
    ''' </summary>
    Private Shared m_KnownFileNames As New List(Of String)


    ''' <summary>
    ''' Event wenn eine neue MP3-Datei erstellt wurde
    ''' </summary>
    ''' <remarks>Geschieht nicht immer im Haupt-Thread der Anwendung</remarks>
    Public Event File_Created(ByVal File As MP3File)

    ''' <summary>
    ''' Event bei Entfernen einer MP3-Datei
    ''' </summary>
    ''' <remarks>Geschieht nicht immer im Haupt-Thread der Anwendung</remarks>
    Public Event File_Removed(ByVal File As MP3File)


    ''' <summary>
    ''' Aktuell ausgewhlte MP3-Datei
    ''' </summary>
    Private m_SelectedFile As MP3File

    ''' <summary>
    ''' Event bei nderung der ausgewhlten MP3-Datei
    ''' </summary>
    ''' <remarks>Geschieht nicht immer im Haupt-Thread der Anwendung</remarks>
    Public Event SelectedFile_Changed()

#End Region

#Region " Constructor "
    ''' <summary>
    ''' Shared Konstruktor
    ''' </summary>
    Shared Sub New()
        ' Globales Ausgabe-Directory fr MP3-Dateien festlegen
        m_OutputPath = Application.StartupPath & "\output"
        ' wenn es nicht existiert, dieses Directory anlegen
        If Not IO.Directory.Exists(m_OutputPath) Then
            IO.Directory.CreateDirectory(m_OutputPath)
        End If

        Dim FileNames() As String = IO.Directory.GetFiles(m_OutputPath)
        For Each txt As String In FileNames
            Dim FN As String = IO.Path.GetFileNameWithoutExtension(txt)
            If Not m_KnownFileNames.Contains(FN) Then
                m_KnownFileNames.Add(FN)
            End If
        Next
    End Sub

    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    Public Sub New()

    End Sub

    ''' <summary>
    ''' Konstruktor fr das Lesen einer XML-Datei
    ''' </summary>
    Public Sub New(ByVal Nd As XmlNode)
        ' Im globalen Ausgabe-Directory vorhandene Dateinamen in eine Liste einfgen

        For Each xNd As XmlNode In Nd.ChildNodes
            Dim NewFile As New MP3File(xNd, Me)
            MyBase.Add(NewFile)
        Next
        If Me.Count > 0 Then
            m_SelectedFile = Me(0)
        End If
    End Sub
#End Region

#Region " Properties "

    ''' <summary>
    ''' Aktuell ausgewhlte MP3-Datei
    ''' </summary>
    Property SelectedFile() As MP3File
        Get
            Return m_SelectedFile
        End Get
        Set(ByVal value As MP3File)
            If m_SelectedFile IsNot value Then
                m_SelectedFile = value
                RaiseEvent SelectedFile_Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Pfad zum global gltigen Ausgabe-Directory fr aufgezeichnete MP3-Dateien
    ''' </summary>
    Shared ReadOnly Property OutputPath() As String
        Get
            Return m_OutputPath
        End Get
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' schreibt den Inhalt der MP3-Datei in eine XML-Datei
    ''' </summary>
    ''' <param name="Writer"></param>
    ''' <remarks></remarks>
    Public Sub Write_XML(ByVal Writer As XmlTextWriter)
        For Each File As MP3File In Me
            Writer.WriteStartElement("MP3File")
            File.Write_XML(Writer)
            Writer.WriteEndElement()
        Next
    End Sub


    ''' <summary>
    ''' liefert eine neue MP3-Datenquelle mit neuem Dateinamen zum Aufzeichnen von Daten
    ''' </summary>
    Public Function CreateReceiveFile(ByVal OrigStationName As String) As MP3ReceivingFile
        ' Warten, bis der Eintritt sicher ist
        m_Mutex.WaitOne()

        Dim StationName As String = Create_FileName(OrigStationName)

        ' Zhler fr Datei-Endungen
        Dim Counter As Integer = 0
        Dim FileName As String = StationName
        While m_KnownFileNames.Contains(FileName)
            Counter += 1
            FileName = StationName & Counter.ToString
        End While
        ' neuen Dateinamen zur Liste mit bekannten Dateinamen hinzufgen
        m_KnownFileNames.Add(FileName)

        ' Mutex freigeben
        m_Mutex.ReleaseMutex()
        ' Dateinamen zurck geben

        Dim File As New MP3ReceivingFile(FileName, Me, OrigStationName)
        Me.Add(File)

        Return File
    End Function

    ''' <summary>
    ''' liefert einen geeigneten Dateinamen ohne unerlaubte Zeichen
    ''' </summary>
    Private Shared Function Create_FileName(ByVal Original As String) As String
        Dim Result As String = Original
        For Each Ch As Char In IO.Path.GetInvalidFileNameChars
            Result = Result.Replace(Ch.ToString, " ")
        Next

        While Result.IndexOf("  ") > -1
            Result = Result.Replace("  ", " ")
        End While

        Return Result.Trim
    End Function

    ''' <summary>
    ''' berschriebene Methode fr das Hinzufgen einer MP3-Datei
    ''' </summary>
    Public Shadows Sub Add(ByVal item As MP3File)
        MyBase.Add(item)
        RaiseEvent File_Created(item)
        Me.SelectedFile = item
    End Sub

    ''' <summary>
    ''' berschriebene Methode fr das Entfernen einer MP3-Datei
    ''' </summary>
    Public Shadows Function Remove(ByVal item As MP3File) As Boolean
        Dim Result As Boolean = False

        If IO.File.Exists(m_OutputPath & "\" & item.FileName & ".mp3") Then
            Try
                IO.File.Delete(m_OutputPath & "\" & item.FileName & ".mp3")
                Dim Idx As Integer = Me.IndexOf(item)
                If Idx > 0 Then Idx -= 1
                Result = MyBase.Remove(item)
                RaiseEvent File_Removed(item)
                If Me.Count > Idx Then
                    Me.SelectedFile = Me(Idx)
                Else
                    Me.SelectedFile = Nothing
                End If
            Catch ex As Exception

            End Try
        End If

        Return Result
    End Function

#End Region

End Class


