Imports System.Xml
Imports System.Text.RegularExpressions
Imports System.IO

''' <summary>
''' Liste mit verwendeten Radio-Stationen
''' </summary>
Public Class RadioStationSource
    Inherits List(Of RadioStation)

#Region " Fields / Events "

    ''' <summary>
    ''' Referenz zur Sammlung verwendeter MP3-Dateien
    ''' </summary>
    ''' <remarks>Diese Liste ermglicht das Erstellen neuer MP3-Dateien zum Empfang</remarks>
    Private m_MP3Source As MP3Source

    ''' <summary>
    ''' Globale Liste mit bekannten Radiostations-URLs
    ''' </summary>
    ''' Liste mit bereits bekannten URLs 
    Private m_KnownURLs As New List(Of String)

    ''' <summary>
    ''' Event nach dem Erstellen einer neuen Radio-Station 
    ''' </summary>
    ''' <param name="Station">Radio-Station</param>
    Public Event StationAdded(ByVal Station As RadioStation)

    ''' <summary>
    ''' Event nach dem Entfernen einer Radio-Station 
    ''' </summary>
    ''' <param name="Station">Radio-Station</param>
    Public Event StationRemoved(ByVal Station As RadioStation)

    ''' <summary>
    ''' aktuell ausgewhlte Radio-Station
    ''' </summary>
    Private m_SelectedStation As RadioStation

    ''' <summary>
    ''' Event bei nderung der ausgewhlten Radio-Station
    ''' </summary>
    Public Event SelectedStation_Changed()

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Default Konstruktor
    ''' </summary>
    Public Sub New(ByVal Source As MP3Source)
        m_MP3Source = Source
    End Sub

    ''' <summary>
    ''' Konstruktor zum Erstellen einer Radio-Station beim Lesen einer XML-Datei
    ''' </summary>
    Public Sub New(ByVal Nd As XmlNode, ByVal Source As MP3Source)
        m_MP3Source = Source
        For Each xNd As XmlNode In Nd.ChildNodes
            Dim Rec As New RadioStation(xNd, m_MP3Source)
            MyBase.Add(Rec)
            m_KnownURLs.Add(Rec.URL)
        Next
        If Me.Count > 0 Then
            m_SelectedStation = Me(0)
        End If
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' Aktuell ausgewhlte Radiostation
    ''' </summary>
    Property SelectedStation() As RadioStation
        Get
            Return m_SelectedStation
        End Get
        Set(ByVal value As RadioStation)
            If m_SelectedStation IsNot value Then
                m_SelectedStation = value
                RaiseEvent SelectedStation_Changed()
            End If
        End Set
    End Property

    ''' <summary>
    ''' Liste mit bereits bekannten URLs
    ''' </summary>
    ReadOnly Property KnownURLs() As List(Of String)
        Get
            Return m_KnownURLs
        End Get
    End Property


#End Region

#Region " Methods "

    ''' <summary>
    ''' beendet alle Threads zum Empfang von Daten
    ''' </summary>
    Public Sub QuitAllReceiving()
        For Each Rec As RadioStation In Me
            Rec.StopAllRecording()
        Next
    End Sub

    ''' <summary>
    ''' Fgt die zu einer URL gehrenden Radiostationen zur Auflistung hinzu
    ''' </summary>
    Public Sub AddStation(ByVal URL As String)
        If URL.Trim <> "" Then
            ' wenn als URL eine PlayList bergeben wurde, enthaltene 
            ' RadioStation-URLs extrahieren
            If URL.Trim.ToLower.EndsWith(".pls") OrElse URL.Trim.ToLower.EndsWith(".m3u") Then
                Try
                    Dim client As New System.Net.WebClient()
                    Dim data As Byte() = client.DownloadData(URL)
                    If data Is Nothing Then Return

                    Dim strData As String = System.Text.Encoding.ASCII.GetString(data, 0, data.Length)
                    Dim URLs() As String = GetURLsFromPlayList(strData)
                    For Each txt As String In URLs
                        If txt <> "" Then
                            AddSingleStation(txt)
                        End If
                    Next
                Catch ex As Exception

                End Try

            Else
                ' ansonsten die URL zur Liste hinzu fgen
                AddSingleStation(URL)
            End If
        End If
    End Sub

    ''' <summary>
    ''' Liefert eventuell vorhandene URLs aus einer PlayList
    ''' </summary>
    Private Function GetURLsFromPlayList(ByVal txt As String) As String()
        Dim Result As New List(Of String)

        Using Reader As New IO.StringReader(txt)
            Dim line As String = Reader.ReadLine

            If txt.IndexOf("#EXTM3U") > -1 Then
                Dim regex As New Regex("^#EXTINF:", RegexOptions.IgnoreCase)
                While line IsNot Nothing
                    If regex.Match(line).Success Then
                        line = Reader.ReadLine
                        If line IsNot Nothing Then
                            Result.Add(line.Trim)
                        End If
                    End If
                    line = Reader.ReadLine
                End While
            ElseIf txt.IndexOf("[playlist]") > -1 Then
                Dim regex As New Regex("^File\d{1,}=", RegexOptions.IgnoreCase)
                Dim index As Integer = 0
                While line IsNot Nothing
                    If regex.Match(line).Success Then
                        line = line.ToLower
                        index += 1
                        Result.Add(SharedFunctions.GetSubString(line, "file" & index & "=", "").Trim)
                    End If
                    line = Reader.ReadLine
                End While
            Else
                While line IsNot Nothing
                    Result.Add(line.Trim)
                    line = Reader.ReadLine
                End While
            End If
        End Using

        Return Result.ToArray
    End Function

    ''' <summary>
    ''' Fgt eine neue Radio-Station zu dieser Liste hinzu und liefert ein Event
    ''' </summary>
    Private Sub AddSingleStation(ByVal URL As String)
        If Not m_KnownURLs.Contains(URL) Then
            Dim NewStation As New RadioStation(URL, m_MP3Source)
            Me.Add(NewStation)
        End If
    End Sub

    ''' <summary>
    ''' Fgt eine neue Radio-Station zu dieser Liste hinzu und liefert ein Event
    ''' </summary>
    Public Sub AddSingleStation(ByVal URL As String, ByVal StationName As String)
        If Not m_KnownURLs.Contains(URL) Then
            Dim NewStation As New RadioStation(URL, m_MP3Source)
            NewStation.StationName = StationName
            Me.Add(NewStation)
        End If
    End Sub

    ''' <summary>
    ''' schreibt Inhalt und Beschreibung aller Radio-Stationen in eine XML-Datei
    ''' </summary>
    Public Sub Write_XML(ByVal Writer As XmlTextWriter)
        For Each Station As RadioStation In Me
            Writer.WriteStartElement("RadioStation")
            Station.Write_XML(Writer)
            Writer.WriteEndElement()
        Next
    End Sub

    ''' <summary>
    ''' Fgt der Auflistung eine Radiostation hinzu
    ''' </summary>
    ''' <param name="item">RadioStation</param>
    Public Shadows Sub Add(ByVal item As RadioStation)
        MyBase.Add(item)
        m_KnownURLs.Add(item.URL)
        RaiseEvent StationAdded(item)
        Me.SelectedStation = item
    End Sub


    ''' <summary>
    ''' entfernt eine Radiostation
    ''' </summary>
    ''' <param name="item">RadioStation</param>
    Public Shadows Function Remove(ByVal item As RadioStation) As Boolean
        Dim Result As Boolean = False

        Dim Idx As Integer = Me.IndexOf(item)
        If Idx > 0 Then Idx -= 1
        item.IsRecording = False
        m_KnownURLs.Remove(item.URL)
        Result = MyBase.Remove(item)
        RaiseEvent StationRemoved(item)
        If Me.Count > Idx Then
            Me.SelectedStation = Me(Idx)
        Else
            Me.SelectedStation = Nothing
        End If

        Return Result
    End Function
#End Region

End Class
