Imports System.Threading

''' <summary>
''' Radio-Empfnger
''' </summary>
''' <remarks>steuert den die Aufzeichnung von MP3-Streams fr eine Radiostation in einem separaten Thread</remarks>
Public Class RadioStationReceiver
    Implements IDisposable

#Region " Fields / Events "

    ''' <summary>
    ''' Referenz zur Radiostation
    ''' </summary>
    Private m_Station As RadioStation

    ''' <summary>
    ''' verwendetes Objekt mit FileStream und MP3-Dateibeschreibung, in dem die aufgezeichneten Daten gespeichert werden
    ''' </summary>
    Private WithEvents m_ReceiveFile As MP3ReceivingFile

    ''' <summary>
    ''' Objekt zum Sammeln empfangener Bytes und Daten
    ''' </summary>
    Private WithEvents m_ReceivedByteList As ReceivedByteList

    ''' <summary>
    ''' Empfnger fr MP3-Streams aus dem Internet
    ''' </summary>
    Private WithEvents m_StreamReceiver As StreamReceiver

    ''' <summary>
    ''' Event nach Erstellen einer neuen MP3-Empfangsdatei
    ''' </summary>
    Public Event File_Created()

    ''' <summary>
    ''' Event bei nderung des Empfangsstatus
    ''' </summary>
    Public Event ReceiveState_Changed()

    ''' <summary>
    ''' Event bei Fehler whrend des Empfangs
    ''' </summary>
    Public Event Was_Error()

    ''' <summary>
    ''' Event bei nderung des aktuellen Musiktitels
    ''' </summary>
    Public Event CurrentTitle_Changed()

    ''' <summary>
    ''' Flag zum Verhindern von Events bei Dispose
    ''' </summary>
    Private m_Disposing As Boolean = False

#End Region

#Region " Constructor "

    ''' <summary>
    ''' Default Konstruktor
    ''' </summary>
    Public Sub New(ByVal Station As RadioStation)
        m_Station = Station
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' aktueller Empfangsstatus
    ''' </summary>
    ReadOnly Property ReceiveState() As ReceiveStates
        Get
            If m_StreamReceiver Is Nothing Then
                Return ReceiveStates.Inactive
            Else
                Return m_StreamReceiver.ReceiveState
            End If
        End Get
    End Property

    ''' <summary>
    ''' aktueller Musiktitel
    ''' </summary>
    ReadOnly Property CurrentTitle() As String
        Get
            If m_ReceiveFile Is Nothing Then
                Return ""
            Else
                Return m_ReceiveFile.CurrentTitle
            End If
        End Get
    End Property

#End Region

#Region " Methods "

    ''' <summary>
    ''' Startet dem Empfang von Daten aus einem MP3-Stream in einem neuen Thread
    ''' </summary>
    ''' <param name="Url">Url zum Streaming-Server</param>
    ''' <param name="MustCreateNewFile">Flag, ob eine neue MP3-Datei erstellt werden muss</param>
    Public Sub Execute(ByVal Url As String, ByVal MustCreateNewFile As Boolean)

        ' Wenn schon ein Empfnger mit eigenem Socket existiert, diesen frei geben
        If m_StreamReceiver IsNot Nothing Then m_StreamReceiver.Dispose()

        ' Wenn eine neue MP3-Strukturinformation erstellt werden muss
        If MustCreateNewFile OrElse m_ReceiveFile Is Nothing Then
            If m_ReceiveFile IsNot Nothing Then
                ' ggf. noch offene MP3-Strukturinformation (und FileStream) schliessen
                m_ReceiveFile.CloseFile()
            End If
            ' Neue MP3-Strukturinformation (mit FileStream) erstellen
            m_ReceiveFile = m_Station.MP3Source.CreateReceiveFile(m_Station.StationName)
            ' Event zur nderung der Textanzeige liefern
            RaiseEvent File_Created()
        End If

        ' Objekt zum Sammeln empfangener Daten erstellen
        m_ReceivedByteList = New ReceivedByteList(m_Station.StationName)
        ' neuen Empfnger mit Socket erstellen
        m_StreamReceiver = New StreamReceiver(Url, m_ReceivedByteList, m_ReceiveFile)

        ' Thread zum Empfang starten
        Dim ReceiveThread As New Thread(New ThreadStart(AddressOf m_StreamReceiver.Receive))
        ReceiveThread.IsBackground = True
        ReceiveThread.Start()
    End Sub


    ''' <summary>
    ''' Einbinden von IDisposable
    ''' </summary>
    Public Sub Dispose() Implements IDisposable.Dispose
        m_Disposing = True
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

    ''' <summary>
    ''' Einbinden von IDisposable
    ''' </summary>
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If m_ReceivedByteList IsNot Nothing Then
                m_ReceivedByteList.IsValid = False
            End If

            If m_StreamReceiver IsNot Nothing Then
                m_StreamReceiver.Dispose()
            End If
            If m_ReceiveFile IsNot Nothing Then
                m_ReceiveFile.Dispose()
            End If
        End If
    End Sub

    ''' <summary>
    ''' Handler bei nderung des empfangenen Radiostations-Namens
    ''' </summary>
    Private Sub m_ReceivedContents_StationName_Changed() Handles m_ReceivedByteList.StationName_Changed
        If m_Disposing Then Exit Sub
        m_Station.StationName = m_ReceivedByteList.StationName
        If m_ReceiveFile IsNot Nothing Then
            m_ReceiveFile.StationName = m_ReceivedByteList.StationName
        End If
    End Sub

    ''' <summary>
    ''' Handler fr eine Redirect-Information beim Empfang vom Server
    ''' </summary>
    ''' <param name="NewURL">Neue URL</param>
    Private Sub m_StreamReceiver_MustRedirect(ByVal NewURL As String) Handles m_StreamReceiver.MustRedirect
        If m_Disposing Then Exit Sub
        Me.Execute(NewURL, False)
    End Sub

    ''' <summary>
    ''' Handler bei ndern des Empfangs-Status des StraemReceiver
    ''' </summary>
    Private Sub m_StreamReceiver_ReceiveState_Changed() Handles m_StreamReceiver.ReceiveState_Changed
        If m_Disposing Then Exit Sub
        RaiseEvent ReceiveState_Changed()
    End Sub

    ''' <summary>
    ''' Handler bei auftreten eines Fehlers whrend des Empfangs im StraemReceiver
    ''' </summary>
    Private Sub m_StreamReceiver_Was_Error() Handles m_StreamReceiver.Was_Error
        If m_Disposing Then Exit Sub
        RaiseEvent Was_Error()
    End Sub

    ''' <summary>
    ''' Handler bei nderung des aktuellen Titels in der MP3-Datei
    ''' </summary>
    Private Sub m_MP3FileInfo_CurrentTitle_Changed() Handles m_ReceiveFile.CurrentTitle_Changed
        If m_Disposing Then Exit Sub
        RaiseEvent CurrentTitle_Changed()
    End Sub

#End Region

End Class

